//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

// The AIPlayer C++ class implements the following methods.
//
// AIPlayer::stopMove();
//    Stop the player where he is.
//
// AIPlayer::clearAim();
//    Clear any current aim location or aim object.  The player will
//    maintain his current orientation, or face towards his destination
//    if moving.
//
// AIPlayer::setMoveSpeed( float );
//    Set the speed (0-1) at which the player moves.
//
// AIPlayer::setMoveDestination( "x y z" );
//    Set the destination point to move towards. The Z location is ignored
//    though the player will face upwards (or downards) towards the destination
//    if no other aim target is set.
//
// AIPlayer::getMoveDestination();
//    Returns the current destination point.
//
// AIPlayer::setAimLocation( \"x y z\" );
//    Set a point to look at.  The player maintains his orientation towards
//    this point even while moving towards a destination point.
//
// AIPlayer::setAimObject( obj );
//    Set an object to look at.  The player maintains his orientation towards
//    this object, even while moving towards a destination point. The aim
//    object can also be moving.
//
// AIPlayer::getAimLocation();
//    Returns the current point the player is looking at. If the player has
//    an aimObject set, this would be the current position of the target
//    object.
//
// AIPlayer::getAimObject();
//    Returns the current target object, if there is one.

//-----------------------------------------------------------------------------
// Callback Handlers
//-----------------------------------------------------------------------------

$BotCount = 0;
$GuardBot = 1;

function Armor::onStuck(%this,%obj) 
{
   // Invoked if the player is stuck while moving
   // This method is currently not being invoked correctly.
   echo( "I'm stuck" );
}

function Armor::onReachDestination(%this,%obj)
{
   // Invoked when the player arrives at his destination point.
   echo( "onReachDestination" );
}

function Armor::onTargetEnterLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // that object becomes visible.
   %obj.setImageTrigger($RightHandSlot,true);
   //echo( "onTargetEnterLOS" );
}

function Armor::onTargetExitLOS(%this,%obj)
{
   // If an aim target object is set, this method is invoked when
   // the object is not longer visible.
   //echo( "onTargetExitLOS" );
   %obj.setImageTrigger($RightHandSlot,false);
   %obj.TargetObj = 0;
}


//-----------------------------------------------------------------------------

function AIPlayer::spawnPlayer(%pos, %name, %type)
{
   // An example function which creates a new AIPlayer object
   // using the the example player datablock.
$BotCount++;   
$Bot[$BotCount] = new AIPlayer() {
      dataBlock = LightMaleHumanArmor;
      aiPlayer = true;
   };
   MissionCleanup.add($Bot[$BotCount]);

   // Player setup
   $Bot[$BotCount].setMoveSpeed(1);
   $Bot[$BotCount].setTransform(%pos);
   $Bot[$BotCount].setEnergyLevel(60);
   $Bot[$BotCount].setShapeName(%name);
   $Bot[$BotCount].mountimage("shieldimage",$LeftHandSlot);
   $Bot[$BotCount].mountimage("bowimage",$RightHandSlot);
   $Bot[$BotCount].mountimage("platemailImage",$BackSlot);
   $Bot[$BotCount].TargetObj = 0;
   $Bot[$BotCount].Type = %type;

   $Bot[$BotCount].OrignialRotation = $Bot[$BotCount].GetAimLocation();
   BotThink($BotCount);
}   

function BotThink(%bot)
{
	
	switch($Bot[%bot].type)
	{
	case $GuardBot:
		if(isObject($Bot[%bot].targetObj))
		{
			$Bot[%bot].setAimObject($Bot[%bot].targetObj);
			//$Bot[%bot].setImageTrigger($RightHandSlot,true);
		}
		else
		{
			$Bot[%bot].clearAim();
			//$Bot[%bot].setAimLocation($Bot[$BotCount].OrignialRotation);
			$Bot[%bot].setImageTrigger($RightHandSlot,false);
		}
	case $PassiveBot:
	}
	schedule(500,0,"BotThink",%bot);
}


function AIplayer::jump(%this)
{
	%this.setImageTrigger(2,true);
	schedule( 100, 0, endjump, %this );
}

function endjump(%bot)
{
	%bot.setImageTrigger(2, false);
}
