// Created by Rick
// rick@gibbed.us
// http://rick.gibbed.us/

// Version: 1.4

// Mission-oriented block persistence for Blockland.

// Copyright (c) 2005 gibbed.us
// 
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the
// use of this software.
// 
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it
// freely, subject to the following restrictions:
// 
// 1. The origin of this software must not be misrepresented; you must not claim
//    that you wrote the original software. If you use this software in a product,
//    an acknowledgment in the product documentation would be appreciated but is
//    not required.
// 
// 2. Altered source versions must be plainly marked as such, and must not be
//    misrepresented as being the original software.
// 
// 3. This notice may not be removed or altered from any source distribution.

function loadPersistence(%client,$UniquePersistName)
{
	messageAll("MsgPersistence", "\c2Blocks are being loaded from " @ $UniquePersistName @ ".persistence. Please be patient...");
	%filename = $Server::MissionFile @ "_" @ $UniquePersistName @ ".persistence";
	echo("Loading persistence file " @ %filename);
	exec(%filename);

	$TotalCopSpawnPoints = 0;
	$TotalRobberSpawnPoints = 0;
$ServerGroup = 0;
for (%i = 0; %i < MissionCleanup.getCount(); %i++)
		{
			%brick = MissionCleanup.getObject(%i);

			if(%brick.isDoor)
			{
				if(%brick.group > $ServerGroup)
				{
					$ServerGroup = %brick.group;

				}
				$Movers[$TotalMoversPlaced++] = %brick;
			}

			if(%brick.isCopSpawn)
			{
				$CopSpawn[$TotalCopSpawnPoints++] = %brick;
			}
			if(%brick.isRobberSpawn)
			{
				$RobberSpawn[$TotalRobberSpawnPoints++] = %brick;
			}
			if(%brick.flameemitter $= 1)
			{
			   %curtrans = %brick.getTransform();
			   %curx = getword(%curtrans,0);
		           %cury = getword(%curtrans,1);
		           %curz = getword(%curtrans,2);
			   %newX = %curx + 0.5;
			   %newY = %cury + 0.5;
			   %newZ = %curz + 0.2;
			   %newTrans = %newX SPC %newY SPC %newZ;
			   %brick.flameEmitter = new ParticleEmitterNode(brickFireNode) {
     			   position = %newTrans;
      			   rotation = "1 0 0 0";
      			   scale = "1 1 1";
      			   dataBlock = "FireParticleEmitterNode";
     		   	   emitter = "FireParticleEmitter";
      			   velocity = "1.0";
   			   };
			   %brick.smokeEmitter = new ParticleEmitterNode(brickSmokeNode) {
     			   position = %newTrans;
      			   rotation = "1 0 0 0";
      			   scale = "1 1 1";
      			   dataBlock = "SmokeParticleEmitterNode";
     		   	   emitter = "SmokeParticleEmitter";
      			   velocity = "1.0";
   			   };
			}
			if(%brick.isBaseTrigger)
			{
				%size = %brick.TriggerSize;
				%trans = %brick.getWorldBoxCenter();
				%x = getWord(%trans,0)-(getword(%size,0)/2);
				%y = getWord(%trans,1)+(getword(%size,1)/2);
				%z = getWord(%trans,2);
				%brick.BaseTrigger = new Trigger() {
     				position = %x SPC %y SPC %z;
      				rotation = "1 0 0 0";
      				scale = %brick.TriggerSize;
      				dataBlock = "DepositTrigger";
      				polyhedron = "0.0000000 0.0000000 0.0000000 1.0000000 0.0000000 0.0000000 0.0000000 -1.0000000 0.0000000 0.0000000 0.0000000 1.0000000";
   				};
			}
			if(%brick.isBankTrigger)
			{
				%size = %brick.TriggerSize;
				%trans = %brick.getWorldBoxCenter();
				%x = getWord(%trans,0)-(getword(%size,0)/2);
				%y = getWord(%trans,1)+(getword(%size,1)/2);
				%z = getWord(%trans,2);
				%brick.BankTrigger = new Trigger() {
     				position = %x SPC %y SPC %z;
      				rotation = "1 0 0 0";
      				scale = %brick.TriggerSize;
      				dataBlock = "StealTrigger";
      				polyhedron = "0.0000000 0.0000000 0.0000000 1.0000000 0.0000000 0.0000000 0.0000000 -1.0000000 0.0000000 0.0000000 0.0000000 1.0000000";
   				};
			}

		}

	if($Pref::Server::LoadPersistenceWithOwnerships)
	{
		for (%i = 0; %i < MissionCleanup.getCount(); %i++)
		{
			for(%t = 0; %t < ClientGroup.getCount(); %t++)
			{
				%brick = MissionCleanup.getObject(%i);
				%cl = ClientGroup.getObject(%t);
				if(%brick.OwnerIP $= getRawIP(%cl))
				{
					%brick.Owner = %cl;
					%brick.OwnerAway = 0;
				}
			}
		}
	}
	else
	{
		for (%i = 0; %i < MissionCleanup.getCount(); %i++)
		{
			%brick = MissionCleanup.getObject(%i);
			%brick.Owner = %client;
		}
	}
	//%testy = schedule(1000 * 60 * 1, 0, "saveSchedule");
	//saveSchedule();
}

function savePersistence($UniquePersistName)
{
	messageAll("MsgPersistence", "\c2Blocks are being saved to " @ $UniquePersistName @ ".persistence. Please be patient...");
	saveBlocks("./" @ $Server::MissionFile @ "_" @ $UniquePersistName @ ".persistence");
	//saveLists();
}

function saveBlocks(%filename)
{
	// Let's add some brick "persistence".
	%file = new FileObject();
	%file.openForWrite(%filename);
	
	// Generate script which searches for
	// baseplates in the level and assigns them to variables.
	// It also sets the baseplate color. :)

	%file.writeLine("// Add Extra Map Items");
	//%file.writeLine("$TotalAddedMapItems = 0;");

	for(%t = 0; %t < $TotalAddedMapItems; %t++)
	{
		
		if($AddedMapItems[%t].dataBlock $= "baseplate32")
		{
			%file.writeLine("$AddedMapItems[$TotalAddedMapItems] = new StaticShape()");
			%file.writeLine("{");
			%file.writeLine("\tposition = \"" @ $AddedMapItems[%t].position @ "\";");
			%file.writeLine("\trotation = \"" @ $AddedMapItems[%t].rotation @ "\";");
			%file.writeLine("\tscale = \"" @ $AddedMapItems[%t].scale @ "\";");
			%file.writeLine("\tdatablock = \""@$AddedMapItems[%t].Datablock@"\";");
			%file.writeLine("};");
			%file.writeLine("MissionGroup.add($AddedMapItems[$TotalAddedMapItems]);");
			%file.writeLine("$TotalAddedMapItems++;");
		}
		if($AddedMapItems[%t].dataBlock $= "gray32")
		{
			%file.writeLine("$AddedMapItems[$TotalAddedMapItems] = new StaticShape()");
			%file.writeLine("{");
			%file.writeLine("\tposition = \"" @ $AddedMapItems[%t].position @ "\";");
			%file.writeLine("\trotation = \"" @ $AddedMapItems[%t].rotation @ "\";");
			%file.writeLine("\tscale = \"" @ $AddedMapItems[%t].scale @ "\";");
			%file.writeLine("\tdatablock = \""@$AddedMapItems[%t].Datablock@"\";");
			%file.writeLine("};");
			%file.writeLine("MissionGroup.add($AddedMapItems[$TotalAddedMapItems]);");
			%file.writeLine("$TotalAddedMapItems++;");
		}
		else
		{
			%file.writeLine("$AddedMapItems[$TotalAddedMapItems] = new Item()");
			%file.writeLine("{");
			%file.writeLine("\tposition = \"" @ $AddedMapItems[%t].position @ "\";");
			%file.writeLine("\trotation = \"" @ $AddedMapItems[%t].rotation @ "\";");
			%file.writeLine("\tscale = \"" @ $AddedMapItems[%t].scale @ "\";");
			%file.writeLine("\tdatablock = \"" @ $AddedMapItems[%t].Datablock @ "\";");
			%file.writeLine("\tcollideable = \"" @ $AddedMapItems[%t].collideable @ "\";");
			%file.writeLine("\tstatic = \"" @ $AddedMapItems[%t].static @ "\";");
			%file.writeLine("\trotate = \"" @ $AddedMapItems[%t].rotate @ "\";");
			%file.writeLine("};");
			%file.writeLine("MissionGroup.add($AddedMapItems[$TotalAddedMapItems]);");
			%file.writeLine("if($Pref::Server::ItemsCostMoney)");
			%file.writeLine("{");
%file.writeLine("if($AddedMapItems[$TotalAddedMapItems].Datablock.Cost > 0)");
%file.writeLine("{");
%file.writeLine("$AddedMapItems[$TotalAddedMapItems].setShapeName(\"$\"@$AddedMapItems[$TotalAddedMapItems].Datablock.Cost);");
%file.writeLine("}");				
%file.writeLine("}");	
			
			%file.writeLine("$TotalAddedMapItems++;");
		}
	}
	
	%file.writeLine("// Search for baseplates");
	
	%file.writeLine("for (%i = 0; %i < MissionGroup.getCount(); %i++)");
	%file.writeLine("{");
	%file.writeLine("\t%obj = MissionGroup.getObject(%i);");
	
	%file.writeLine("\tif (%obj.dataBlock $= \"gray32\")");
	%file.writeLine("\t{");
	
	%count = MissionGroup.getCount();
	%first = true;
	for (%i = 0; %i < %count; %i++)
	{
		%obj = MissionGroup.getObject(%i);
	
		if (%obj.dataBlock $= "gray32")
		{
			if (%first)
			{
				%file.writeLine("\t\tif (%obj.position $= \"" @ %obj.position @ "\")");
				%file.writeLine("\t\t{");
				%first = false;
			}
			else
			{
				%file.writeLine("\t\telse if (%obj.position $= \"" @ %obj.position @ "\")");
				%file.writeLine("\t\t{");
			}
			
			%file.writeLine("\t\t\t%obj.setSkinName(\"" @ %obj.getSkinName() @ "\");");
			%file.writeLine("\t\t\t%block" @ %obj @ " = %obj;");
			%file.writeLine("\t\t}");
		}
	}
	
	%file.writeLine("\t}");
	%file.writeLine("}");
	
	%file.writeLine("");
	
	// Generate script to create bricks and plates
	
	%count = MissionCleanup.getCount();
	%file.writeLine("// Block creation");
	
	for (%i = 0; %i < %count; %i++)
	{
		%block = MissionCleanup.getObject(%i);
		
		if (getSubStr(%block.dataBlock, 0, 10) $= "staticRoad" || getSubStr(%block.dataBlock, 0, 15) $= "staticBaseplate" || getSubStr(%block.dataBlock, 0, 11) $= "staticBrick" || getSubStr(%block.dataBlock, 0, 11) $= "staticPlate" || getSubStr(%block.dataBlock, 0, 11) $= "staticSlope" || getSubStr(%block.dataBlock, 0, 11) $= "staticFence" || getSubStr(%block.dataBlock, 0, 14) $="staticcylinder")
		{
			%file.writeLine("%block" @ %block @ " = new StaticShape()");
			%file.writeLine("{");
			%file.writeLine("\tposition = \"" @ %block.position @ "\";");
			%file.writeLine("\trotation = \"" @ %block.rotation @ "\";");
			%file.writeLine("\tscale = \"" @ %block.scale @ "\";");
			%file.writeLine("\tdataBlock = \"" @ %block.dataBlock @ "\";");
			%file.writeLine("\townerIP = \"" @ %block.ownerIP @ "\";");
			%file.writeLine("\townerAway = \"" @ 1 @ "\";");
			
			if(%block.flameemitter != "")
			{
			%file.writeLine("\t\flameemitter = \"" @ 1 @ "\";");
			}
			
			if(%block.isDoor == 1)
			{
				%file.writeLine("\t\isDoor = \"" @ %block.isDoor @ "\";");
				%file.writeLine("\t\isPrivateDoor = \"" @ %block.isPrivateDoor @ "\";");
				if(%block.isTeamDoor)
				{
					%file.writeLine("\t\isTeamDoor = \"" @ %block.isTeamDoor @ "\";");
					%file.writeLine("\tteamDoor = \"" @ %block.TeamDoor @ "\";");
				}

				%file.writeLine("\t\doorsteps = \"" @ %block.doorsteps @ "\";");
				%file.writeLine("\t\doortime = \"" @ %block.doortime @ "\";");
				%file.writeLine("\t\doorx = \"" @ %block.doorx @ "\";");
				%file.writeLine("\t\doory = \"" @ %block.doory @ "\";");
				%file.writeLine("\t\doorz = \"" @ %block.doorz @ "\";");
				%file.writeLine("\t\doorreturn = \"" @ %block.doorreturn @ "\";");
				%file.writeLine("\t\doorstartTime = \"" @ %block.doorstartTime @ "\";");
				%file.writeLine("\t\isElevator = \"" @ %block.isElevator @ "\";");

				%file.writeLine("\t\doorRot = \"" @ %block.doorRot @ "\";");
				%file.writeLine("\t\doorReturnPause = \"" @ %block.doorReturnPause @ "\";");
				%file.writeLine("\t\doorreturn = \"" @ %block.doorreturn @ "\";");
				%file.writeLine("\tnoCollisionMove = \"" @ %block.noCollisionMove @ "\";");
				%file.writeLine("\tEulerRot = \"" @ %block.EulerRot @ "\";");
				%file.writeLine("\tDoorPassword = \"" @ %block.DoorPassword @ "\";");
				%file.writeLine("\tPassworded = \"" @ %block.Passworded @ "\";");
				%file.writeLine("\tTriggeredDoor = \"" @ %block.TriggeredDoor @ "\";");
				%file.writeLine("\tTriggerDoorID = \"" @ %block.TriggerDoorID @ "\";");
				%file.writeLine("\tisTrigger = \"" @ %block.isTrigger @ "\";");
				%file.writeLine("\tEulerRot = \"" @ %block.EulerRot @ "\";");

				%file.writeLine("\tdoorToggleReturn = \"" @ %block.doorToggleReturn @ "\";");
				%file.writeLine("\t\doorHasReturned = \"" @ %block.doorHasReturned @ "\";");


			}
			if(%block.teleportObj > 0)
			{
				%file.writeLine("\t\ teleportObj = \"" @ %block.teleportObj @ "\";");
			}
			%file.writeLine("};");
			%file.writeLine("MissionCleanup.add(%block" @ %block @ ");");
			$numBlocks++;
			%file.writeLine("%block" @ %block @ ".setSkinName(\"" @ %block.getSkinName() @ "\");");
			%file.writeLine("%block" @ %block @ ".SkinName=\"" @ %block.getSkinName() @ "\";");
			%file.writeLine("%block" @ %block @ ".NoDestroy=\"" @ %block.noDestroy @ "\";");
			%file.writeLine("%block" @ %block @ ".isJail=\"" @ %block.isJail @ "\";");
			%file.writeLine("%block" @ %block @ ".JailMaxCount=\"" @ %block.JailMaxCount @ "\";");
			if(%block.isWCloaked)
			{
				%file.writeLine("%block" @ %block @ ".setcloaked(1)" @ ";");
				%file.writeLine("%block" @ %block @ ".isWCloaked=\"" @ %block.isWCloaked @ "\";");
			}
			%file.writeLine("%block" @ %block @ ".isBankTrigger=\"" @ %block.isBankTrigger @ "\";");
			%file.writeLine("%block" @ %block @ ".isBaseTrigger=\"" @ %block.isBaseTrigger @ "\";");
			%file.writeLine("%block" @ %block @ ".isRobberSpawn=\"" @ %block.isRobberSpawn @ "\";");
			%file.writeLine("%block" @ %block @ ".isCopSpawn=\"" @ %block.isCopSpawn @ "\";");
			%file.writeLine("%block" @ %block @ ".TriggerSize=\"" @ %block.TriggerSize @ "\";");
			%file.writeLine("%block" @ %block @ ".setShapeName(\"" @ %block.getShapeName() @ "\");");
			%file.writeLine("");
		}
	}
	
	%file.writeLine("// Block links");
	for (%i = 0; %i < %count; %i++)
	{
		%block = MissionCleanup.getObject(%i);
		
		if (getSubStr(%block.dataBlock, 0, 11) $= "staticBrick" || getSubStr(%block.dataBlock, 0, 11) $= "staticPlate" || getSubStr(%block.dataBlock, 0, 11) $= "staticSlope" || getSubStr(%block.dataBlock, 0, 11) $= "staticFence" || getSubStr(%block.dataBlock, 0, 14) $="staticcylinder")
		{
			%k = 0;
			for (%j = 0; %j < %block.upSize; %j++)
			{
				if (MissionCleanup.isMember(%block.up[%j]) || %block.up[%j].dataBlock $= "gray32")
				{
					%file.writeLine("%block" @ %block @ ".up[" @ %k @ "] = %block" @ %block.up[%j] @ ";");
					%k++;
				}
			}
			
			%file.writeLine("%block" @ %block @ ".upSize = \"" @ %k @ "\";");
			
			%k = 0;
			for (%j = 0; %j < %block.downSize; %j++)
			{
				if (MissionCleanup.isMember(%block.down[%j]) || %block.down[%j].dataBlock $= "gray32")
				{
					%file.writeLine("%block" @ %block @ ".down[" @ %k @ "] = %block" @ %block.down[%j] @ ";");
					%k++;
				}
			}
			
			%file.writeLine("%block" @ %block @ ".downSize = \"" @ %k @ "\";");
		}
	}
	
	%file.writeLine("// Baseplate links");
	
	%count = MissionGroup.getCount();
	for (%i = 0; %i < %count; %i++)
	{
		%obj = MissionGroup.getObject(%i);
		
		if (%obj.dataBlock $= "gray32")
		{
			%file.writeLine("%i = %block" @ %obj @ ".upSize;");
			for (%j = 0; %j < %obj.upSize; %j++)
			{
				if (isObject(%obj.up[%j]) && MissionCleanup.isMember(%obj.up[%j]))
				{
					%file.writeLine("%block" @ %obj @".up[$i++] = %block" @ %obj.up[%j] @ ";");
				}
			}
			%file.writeLine("%block" @ %obj @ ".upSize = %i;");
			
			%file.writeLine("%i = %block" @ %obj @ ".downSize;");
			for (%j = 0; %j < %obj.downSize; %j++)
			{
				if (isObject(%obj.down[%j]) && MissionCleanup.isMember(%obj.down[%j]))
				{
					%file.writeLine("%block" @ %obj @ ".down[$i++] = %block" @ %obj.down[%j] @ ";");
				}
			}
			%file.writeLine("%block" @ %obj @ ".downSize = %i;");
		}
	}

	// JBM: Save # of blocks
	%file.writeLine("$numBlocks = " @ $numBlocks @ ";");
	
	%file.close();
	
	// A new .persistence file will not be loadable until Blockland restarts,
	// so we force Blockland to regenerate it's filename cache.
	setModPaths(getModPaths());
	echo("Save finished.");

}
