//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// This file contains Weapon and Ammo Class/"namespace" helper methods
// as well as hooks into the inventory system. These functions are not
// attached to a specific C++ class or datablock, but define a set of
// methods which are part of dynamic namespaces "class". The Items
// include these namespaces into their scope using the  ItemData and
// ItemImageData "className" variable.

// All ShapeBase images are mounted into one of 8 slots on a shape.
// This weapon system assumes all primary weapons are mounted into
// this specified slot:
$WeaponSlot = 0;
$War = 0;

//-----------------------------------------------------------------------------
// Weapon Class 
//-----------------------------------------------------------------------------

//generic weapon sounds
datablock AudioProfile(weaponSwitchSound)
{
   filename    = "~/data/sound/weaponSwitch.wav";
   description = AudioClosest3d;
   preload = true;
};


function Weapon::onUse(%this, %player, %InvPosition)
{
   // Default behavoir for all weapons is to mount it into the
   // this object's weapon slot, which is currently assumed
   // to be slot 0
   //if (%obj.getMountedImage($WeaponSlot) != %data.image.getId()) {
   //   %obj.mountImage(%data.image, $WeaponSlot);
   //   if (%obj.client)
   //      messageClient(%obj.client, 'MsgWeaponUsed', '\c0Weapon selected');
   //}

	//if the image is mounted already, unmount it
	//if it isnt, mount it

	%mountPoint = %this.image.mountPoint;
	%mountedImage = %player.getMountedImage(%mountPoint); 
	
	if(%mountedImage)
	{
		//echo(%mountedImage);
		if(%mountedImage == %this.image.getId())
		{
			//our image is already mounted so unmount it
			%player.unMountImage(%mountPoint);
			messageClient(%player.client, 'MsgHilightInv', '', -1);
			%player.currWeaponSlot = -1;
		}
		else
		{
			//something else is there so mount our image
			%player.mountimage(%this.image, %mountPoint,false,$legoColor[%player.client.colorIndex]);

			messageClient(%player.client, 'MsgHilightInv', '', %InvPosition);
			%player.currWeaponSlot = %invposition;
		}
	}
	else
	{
		//nothing there so mount 
		%player.mountimage(%this.image, %mountPoint,false,$legoColor[%player.client.colorIndex]);

		messageClient(%player.client, 'MsgHilightInv', '', %InvPosition);
		%player.currWeaponSlot = %invposition;
	}
}

function Weapon::onPickup(%this, %obj, %shape, %amount)
{
	%player = %shape;
	%data = %player.getDataBlock();
	%client = %player.client;
	if(%player.weaponCount < %data.maxWeapons)
	{
if(%client.WantBuy != 1 && $Pref::Server::ItemsCostMoney $= 1 && %this.cost > 0)
	{
bottomPrint(%client,"Press <B> to Buy this Weapon!",0.5,1);
	}
	if($Pref::Server::ItemsCostMoney $= 1 && %client.money < %this.cost && %client.WantBuy $= 1)
	{
	messageClient(%client, 'MsgNoAfford', 'You can\'t afford this Item!');
	}
		if (($Pref::Server::ItemsCostMoney == 1 && %client.money >= %this.cost && %client.WantBuy == 1)|| $Pref::Server::ItemsCostMoney == 0)
		{	
			//If there is an open inventory slot put it there
			%freeslot = -1;
			for(%i = 0; %i < %data.maxItems; %i++)
			{
				//echo(%i);
				if(%player.inventory[%i] == 0)
				{
					%freeslot = %i;
					break;
				}
			}
			if(%freeslot != -1)
			{
				if (%obj.isStatic())
					%obj.respawn();
				else
					%obj.delete();
	
				%player.weaponCount++;
				%player.inventory[%freeslot] = %this;
				if($Pref::Server::ItemsCostMoney == 1 && %this.cost > 0)
				{
					%client.money -= %this.cost;
					messageclient(%client,"",'\c2Paid $%1 for %2',%this.cost,%this.pickUpName);
					messageClient(%client,'MsgUpdateMoney',"",%client.Money);
				}
 
				// Inform the client what they got.
				if (%client)
					messageClient(%client, 'MsgItemPickup', '', %freeslot, %this.invName);
				return true;
			}
			else
			{
			if (%client)
			messageClient(%client, 'MsgItemPickup', 'No free slots available!');
			}
		}
	}
	else
	{
	if (%client)
	messageClient(%client, 'MsgItemFailPickup', 'You already have enough weapons!');
	}
	

   // The parent Item method performs the actual pickup.
   // For player's we automatically use the weapon if the
   // player does not already have one in hand.
  // if (Parent::onPickup(%this, %obj, %shape, %amount)) {
  //    if (%shape.getClassName() $= "Player" && 
  //          %shape.getMountedImage($WeaponSlot) == 0)  {
  //       %shape.use(%this);
  //    }
  // }
}

function Weapon::onInventory(%this,%obj,%amount)
{
   // Weapon inventory has changed, make sure there are no weapons
   // of this type mounted if there are none left in inventory.
   if (!%amount && (%slot = %obj.getMountSlot(%this.image)) != -1)
      %obj.unmountImage(%slot);
}   


//-----------------------------------------------------------------------------
// Weapon Image Class
//-----------------------------------------------------------------------------
function WeaponImage::onFire(%this,%obj,%slot)
{
	//messageAll( 'MsgClient', 'weapon fired! %1', %this);

	//shoot projectile
	%projectile = %this.projectile;

if(%obj.client.isImprisoned)
{
return;
}

if(%this.item !$= "wand" && %this.item !$= "axe" && %this.item !$= "wrench" && %this.item !$= "sprayCan" && %this.item !$= "letterCan" && %this.item !$= "blackletterCan" && %projectile !$= "brickDeployProjectile" && %projectile !$= "hammerProjectile" && %obj.client.safe == 1)
		{
			return;
		}	

	if(%this.melee)
	{
		%initPos = %obj.getEyeTransform();
		//%muzzleVector = %obj.getEyeVector();
		%muzzleVector = %obj.getMuzzleVector(%slot);
	}
	else
	{
		%initPos = %obj.getMuzzlePoint(%slot);
		%muzzleVector = %obj.getMuzzleVector(%slot);
	}

	
	%objectVelocity = %obj.getVelocity();
	%muzzleVelocity = VectorAdd(
	  VectorScale(%muzzleVector, %projectile.muzzleVelocity),
	  VectorScale(%objectVelocity, %projectile.velInheritFactor));

	%p = new (%this.projectileType)() 
	{
		dataBlock        = %projectile;
		initialVelocity  = %muzzleVelocity;
		initialPosition  = %initPos;
		sourceObject     = %obj;
		sourceSlot       = %slot;
		client           = %obj.client;
	};
	MissionCleanup.add(%p);
	return %p;
}

function WeaponImage::onMount(%this,%obj,%slot)
{
   // Images assume a false ammo state on load.  We need to
   // set the state according to the current inventory.
	commandtoClient(%obj.client,'showBrickImage',"");
	//fixArmReady(%obj);
	if(%this.armReady)
	{
		if(%obj.getMountedImage($LeftHandSlot))
		{
			if(%obj.getMountedImage($LeftHandSlot).armReady)
				%obj.playthread(1, armReadyBoth);
			else
				%obj.playthread(1, armReadyRight);
		}
		else
		{
			%obj.playthread(1, armReadyRight);
		}
	}
	//%thisdata = %this.getDatablock();

	commandtoClient(%obj.client,'showBrickImage',%this.PreviewFileName);

	if(%this.ammo)	//dont check ammo if the image doesnt use ammo
	{
		if (%obj.getInventory(%this.ammo))
		%obj.setImageAmmo(%slot,true);
	}
	else
	{
		%obj.setImageAmmo(%slot,true);
	}


	if(isObject(%obj.tempBrick))
	{	
		if(%this.getClassname() !$= "StaticShape")
		{
		%player = %obj.client.player;
		%location = %player.tempBrick.getTransform();
		%obj.tempBrick.delete();

		%image = %player.getMountedImage(0);	//the thing the guy is holding
		%static = %image.staticShape;
		%ghost = %static;			//ghost is what we're going to deploy

		%d = new StaticShape()
		{
			datablock = %ghost;
		};
		MissionCleanup.add(%d);
		%d.setTransform(%location);
		%d.setScale("1 1 1.1");
		%d.playthread(0, deploy);
		%d.setSkinName('red');
		%d.isBrickGhost = 1;
		%player.tempBrick = %d;
		}
		else
		{
		%obj.tempBrick.delete();
		}

	}


}

function WeaponImage::onUnmount(%this,%obj,%slot)
{
	%obj.playthread(2, root);	//stop arm swinging 

	%leftimage = %obj.getmountedimage($lefthandslot);
	commandtoClient(%obj.client,'showBrickImage',"");
	if(%leftimage)
	{
		if(%leftimage.armready)
		{
			%obj.playthread(1, armreadyleft);
			return;
		}
	}
	%obj.playthread(1, root);
}


//-----------------------------------------------------------------------------
// Ammmo Class
//-----------------------------------------------------------------------------

function Ammo::onInventory(%this,%obj,%amount)
{
   // The ammo inventory state has changed, we need to update any
   // mounted images using this ammo to reflect the new state.
   for (%i = 0; %i < 8; %i++) {
      if ((%image = %obj.getMountedImage(%i)) > 0)
         if (isObject(%image.ammo) && %image.ammo.getId() == %this.getId())
            %obj.setImageAmmo(%i,%amount != 0);
   }
}   
function weaponDamage(%obj,%col,%this,%pos)
{
	if($Pref::Server::Weapons == 1)
	{
		%player = %obj.client.player;
	
		if(%col.getClassName() !$="StaticShape" && %col.getClassName() !$= "Player" && %col.getClassName() !$= "AIPlayer")
		return;
		%colData = %col.getDataBlock();
		%colDataClass = %colData.classname;
		if(%colData.classname !$= "Brick" && %col.getClassName() !$= "Player" && %col.getClassName() !$= "AIPlayer")
		{
			return;
		}
		if(%col.client.safe == 0)
		{
			if (%col.getClassName() $= "Player" || %col.getClassName() $= "AIPlayer")
			{
	if(%col.client.team $= %obj.client.team && $Pref::Server::DMFriendlyFire == 0 && %obj.client.team!$="")
				{
					return;
				}
				%col.targetObj = %player;
				%damagedone = %this.directDamage;
				if(%col.getMountedImage($LeftHandSlot) == shieldImage.getId())
				{
					%damagedone = %damagedone/2;
				}
				if(%col.getMountedImage($HeadSlot) == pointyHelmetImage.getId())
				{	
					%damagedone = %damagedone/2;
				}
				if(%col.getMountedImage($BackSlot) == plateMailImage.getId())
				{
					%damagedone = %damagedone/2;
				}
				if(%col.getMountedImage($HeadSlot) == HelmetImage.getId())
				{
					%damagedone = %damagedone/2;
				}
				
				%col.damage(%obj,%pos,%damagedone,"RifleBullet");
			}
		}
	}

}

function spearProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	if($Pref::Server::Weapons == 1)
	{
		radiusDamage(%obj,%pos,%this.damageRadius,%this.radiusDamage,"RifleBullet",0);
	}

}

function boltsgProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	if($Pref::Server::Weapons == 1)
	{
		radiusDamage(%obj,%pos,%this.damageRadius,%this.radiusDamage,"RifleBullet",0);
	}
}

function arrowProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

$TreeGrow = 1000;
$TreeGrowInc = 1000;
$MaxTreeHits = 20;

function growTree(%pos)
{
	%x = getWord(%pos,0);
	%y = getWord(%pos,1);
	%z = getWord(%pos,2);
	%pos = %x SPC %y SPC %z;
  	%this = new StaticShape() {
     		position = %pos;
      		rotation = "1 0 0 0";
      		scale = "0.1 0.1 0.1";
      		dataBlock = "pineTree";
   	};

	%this.startFade(0, 0, true);
	%this.hide(true);
	%this.totalHits = 0;

	%this.schedule($TreeGrow, "hide", false);
   	%this.schedule($TreeGrow + 100, "startFade", 1000, 0, false);

	for(%t = 1; %t < 11; %t++)
	{
	   	%this.schedule($TreeGrow + ($TreeGrowInc * %t), "setScale",%t/10 SPC %t/10 SPC %t/10);
	}
}

function axeProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	%player = %obj.client.player;
	
	
	%colData = %col.getDataBlock();
	%colDataClass = %colData.classname;
	if(%col.getClassName() !$="StaticShape")
		return;

	if(%colData.classname $= "pineTree") 
	{
		%col.totalHits++;

		if(%col.totalHits >= $MaxTreeHits)
		{
			%obj.client.Money += 5;
			messageClient(%obj.client,'MsgUpdateMoney','',%obj.client.Money);
			%col.startFade(0, 0, true);
			%col.hide(true);
			%col.totalHits = 0;
			%col.setScale("0.1 0.1 0.1");
			%col.schedule($TreeGrow, "hide", false);
   			%col.schedule($TreeGrow + 100, "startFade", 1000, 0, false);
   			for(%t = 1; %t < 11; %t++)
			{
	   			%col.schedule($TreeGrow + ($TreeGrowInc * %t), "setScale",%t/10 SPC %t/10 SPC %t/10);
			}
		}
	}

	if($Pref::Server::Weapons == 1 && %obj.safe == 0)
	{
	
if(%col.getClassName() !$="StaticShape" && %col.getClassName() !$= "Player" && %col.getClassName() !$= "AIPlayer")
		return;

	if(%colData.classname !$= "Brick" && %col.getClassName() !$= "Player" && %col.getClassName() !$= "AIPlayer")
	{
		return;
	}
	if(%col.client.safe == 0)
	{
		if (%col.getClassName() $= "Player" || %col.getClassName() $= "AIPlayer")
		{
			if(%col.client.team $= %obj.client.team && $Pref::Server::DMFriendlyFire == 0 && %obj.client.team!$="")
			{
				return;
			}
			%col.targetObj = %player;
			%damagedone = %this.directDamage;
			if(%col.getMountedImage($LeftHandSlot) == shieldImage.getId())
			{
				%damagedone = %damagedone/2;
			}
			if(%col.getMountedImage($HeadSlot) == pointyHelmetImage.getId())
			{	
				%damagedone = %damagedone/2;
			}
			if(%col.getMountedImage($BackSlot) == plateMailImage.getId())
			{
				%damagedone = %damagedone/2;
			}
			if(%col.getMountedImage($HeadSlot) == HelmetImage.getId())
			{
				%damagedone = %damagedone/2;
			}
			%col.damage(%obj,%pos,%damagedone,"RifleBullet");
			//radiusDamage(%obj,%pos,%this.damageRadius,%damagedone,"RifleBullet",0);
		
		}
		
	}
	}


}

function swordProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function lightsabreProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	if($Pref::Server::CopsAndRobbers)
	{
		if(%obj.client.team $= "Cops" && %col.client.team !$="Cops" && %col.getDamageLevel() >= 50)
		{
			ImprisonPlayer(%obj.client,%col.client);
			%col.damage(%obj,%pos,5,"RifleBullet");
		}
	}
	else
	{
		weaponDamage(%obj,%col,%this,%pos);
	}
}


function ImprisonPlayer(%client,%victim)
{
	if(%client.team $= "Cops" && %victim.team $= "Robbers")
	{
		if(%victim.isImprisoned == 0)
		{
		for(%t = 0; %t < MissionCleanup.GetCount(); %t++)
		{
			%brick = MissionCleanup.getObject(%t);
			if(%brick.isJail == 1)
			{ 
				if(%brick.JailCount < %brick.JailMaxCount)
				{
					if(isobject(%victim.player))
					{
						%brick.JailCount++;
						%victim.isImprisoned = 1;
						%victim.money = 0;
						messageClient(%victim,'MsgUpdateMoney',"",%victim.money);
						%trans = %brick.getWorldBoxCenter();
						%x = getWord(%trans,0);
						%y = getWord(%trans,1);
						%z = getWord(%trans,2)+0.5;
						%victim.player.setTransform(%x SPC %y SPC %z);
						%victim.JailBrick = %brick;
						messageAll("",'\c2%1 was imprisoned by %2',%victim.name,%client.name);
						%victim.player.unMountImage($HeadSlot);
						%victim.player.mountImage(%victim.chestCode , $decalslot, 1, 'jail');
						%victim.player.mountImage(%victim.faceCode , $faceslot, 1, 'smirk2');
						%victim.player.setSkinName('white');
						%totalImprisonedPeople = %totalImprisonedPeople + %brick.JailCount;
						if(%totalImprisonedPeople >= $TotalRobbers)
						{
							messageAll("","\c2COPS WIN!!!");
							messageAll("","\c2Everyone swaps sides!");
							schedule(5000,0,"RestartCR");
						}
					}
					return;
				}
				else
				{
					%totalImprisonedPeople = %totalImprisonedPeople + %brick.JailMaxCount;
					%JailFull = 1;
				}
			}

		}
		if(%JailFull == 1)
		{
			messageAll("","\c2We've run out of jails");
		}
		else
		{
			messageClient(%client,"","\c2There is no jail built...");
		}
		}
		else
		{
			%victim.isImprisoned = 0;
			%victim.player.setTransform(%victim.TeamSpawn.getTransform());
			%victim.JailBrick.JailCount--;
			messageAll("",'\c2%1 was freed by %2',%victim.name,%client.name);
			%victim.player.mountImage($headCode[$RobHat],$HeadSlot,1,'black');
			%victim.player.mountImage(%client.chestCode , $decalslot, 1, 'bandit');
			%victim.player.mountImage(%client.faceCode , $faceslot, 1, 'evil');
			%victim.player.setSkinName('black');
		}	
	
	}
}

function katanaProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function cutlassProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function bulletProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function duallsabreProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function crossbowarrowProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}

function Pickaxeprojectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}
function pistolshotProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}
function blastProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal) 
{ 
	weaponDamage(%obj,%col,%this,%pos);
}


datablock TriggerData(SafeTrigger)
{
   tickPeriodMS = 100;
};

function SafeTrigger::onEnterTrigger(%this,%trigger,%obj)
{

	if($Pref::Server::Weapons == 1)
	{
		%obj.client.safe = 1;
		messageClient(%obj.client,"","\c2You are now in the safe zone");
	}
}

function SafeTrigger::onLeaveTrigger(%this,%trigger,%obj)
{
	if($Pref::Server::Weapons == 1)
	{
		%obj.client.safe = 0;
		messageClient(%obj.client,"","\c2You are now in the warzone");
	}
}

datablock TriggerData(HealthTrigger)
{
   tickPeriodMS = 100;
};

function HealthTrigger::onEnterTrigger(%this,%trigger,%obj)
{
	%client = %obj.client;
	messageClient(%client,"","\c2Entering Health Regeneration Area");
	%client.player.setRepairRate(0.1);
	if(%trigger.safe == 1)
	{
		if($Pref::Server::Weapons == 1)
		{
			%obj.client.safe = 1;
			messageClient(%obj.client,"","\c2You are now in the safe zone");
		}
	}
}

function HealthTrigger::onLeaveTrigger(%this,%trigger,%obj)
{
	%client = %obj.client;
	messageClient(%client,"","\c2Leaving Health Regeneration Area");
	%client.player.setRepairRate(0);
	if(%trigger.safe == 1)
	{
		if($Pref::Server::Weapons == 1)
		{
			%obj.client.safe = 0;
			messageClient(%obj.client,"","\c2You are now in the warzone");
		}
	}
}
